using DrWatson
@quickactivate "StochApprox"
using Revise
using saControlLimits
import saControlLimits: spsaCL
using StatsBase
using ArgParse
using Random, Distributions
using DataFrames
using JLD2
using TimerOutputs

include(srcdir("settings", "settings.jl"))

DrWatson.default_allowed(c) = (Real, String, Symbol, Vector)
DrWatson.savename(S::Settings, suffix::String) = savename(struct2ntuple(S.PROC), suffix)

argTbl = ArgParseSettings()
@add_arg_table argTbl begin
    "--seed", "-s"
        help = "Simulation seed"
end

parsedArgs = parse_args(ARGS, argTbl)
SD = parse(Int64, parsedArgs["seed"])

simsDir = "sims"

abstol = 1e-05
NminSPSA = 300
OPTS = OptimizationSettings(
    hStart = 0.001,
    nAdaptiveGain = 20,
    initialStepSize = [0.2],
    expNumberLossEval = NminSPSA,
    NminSPSA = NminSPSA,
    NfixedSPSA = 100,
    gg = 0.05,
    abstol = abstol,
    gavg = 100,
    NfixedSA = 100,
    maxiterSA=100,
    maxiterSPSA = 100000,
    minPar = [0.01],
    maxPar = [0.99],
    comparison = nothing)


# ---------------- EWMA optimization ---------------- #

rng = MersenneTwister(SD)
deltas = 0.25:0.25:4.0
ARL0s = [50.0, 100.0, 250.0, 370.0]
for ARL0 in ARL0s
    for delta in deltas
        seed = SD
        OPTSLambda = OptimizationSettings(OPTS, minPar = [0.01], maxPar = [0.99], initialStepSize = [0.2])
        Random.seed!(SD)
        PROC = ProcessSettings(par0 = rand(Beta(10,10), 1), ARL0 = ARL0, delta=[delta], seed=seed)
        settings = Settings(OPTSLambda, PROC)
        filename = savename(settings, "jld2")
        filepath = datadir(simsDir, "EWMA", filename)

        if isfile(filepath)
            continue
        else
            opt = spsaCL(settings, runEWMA)
            saveobj = Dict("delta" => delta, "ARL0" => ARL0, "lambda" => opt[:opt][1], "iter" => opt[:iter])
            safesave(filepath, saveobj)
        end
    end
end


rng = MersenneTwister(SD)
# deltas = [0.25, 0.5, 0.75, 1.0]
ARL0s = [100.0]
for ARL0 in ARL0s
    for delta in deltas
        seed = SD
        OPTSLambda = OptimizationSettings(OPTS, minPar = [0.01], maxPar = [0.99], initialStepSize = [0.2], objective=:Median, constraint=:Median)
        Random.seed!(SD)
        PROC = ProcessSettings(par0 = rand(Beta(10,10), 1), ARL0 = ARL0, delta=[delta], seed=seed)
        settings = Settings(OPTSLambda, PROC)

        filename = savename(settings, "med-med.jld2")
        filepath = datadir(simsDir, "EWMA-median", filename)
        if isfile(filepath)
            continue
        else
            # Optimize median-median
            opt_med_med = spsaCL(settings, runEWMA)
            saveobj = Dict("delta" => delta, "ARL0" => ARL0,
            "lambda" => opt_med_med[:opt][1],
            "type" => "median-median"
            )
            safesave(filepath, saveobj)
        end

        # Optimize ARL-median
        OPTSLambda = OptimizationSettings(OPTS, minPar = [0.01], maxPar = [0.99], initialStepSize = [0.2], objective=:ARL, constraint=:Median)
        Random.seed!(SD)
        PROC = ProcessSettings(par0 = rand(Beta(10,10), 1), ARL0 = ARL0, delta=[delta], seed=seed)
        settings = Settings(OPTSLambda, PROC)
        filename = savename(settings, "arl-med.jld2")
        filepath = datadir(simsDir, "EWMA-median", filename)
        if isfile(filepath)
            continue
        else
            # Optimize arl-median
            opt_arl_med = spsaCL(settings, runEWMA)
            saveobj = Dict("delta" => delta, "ARL0" => ARL0,
            "lambda" => opt_arl_med[:opt][1],
            "type" => "arl-median"
            )
            safesave(filepath, saveobj)
        end


        OPTSLambda = OptimizationSettings(OPTS, minPar = [0.01], maxPar = [0.99], initialStepSize = [0.2], objective=:Median, constraint=:ARL)
        Random.seed!(SD)
        PROC = ProcessSettings(par0 = rand(Beta(10,10), 1), ARL0 = ARL0, delta=[delta], seed=seed)
        settings = Settings(OPTSLambda, PROC)
        filename = savename(settings, "med-arl.jld2")
        filepath = datadir(simsDir, "EWMA-median", filename)
        if isfile(filepath)
            continue
        else
            # Optimize arl-median
            opt_med_arl = spsaCL(settings, runEWMA)
            saveobj = Dict("delta" => delta, "ARL0" => ARL0,
            "lambda" => opt_med_arl[:opt][1],
            "type" => "median-arl"
            )
            safesave(filepath, saveobj)
        end

        OPTSLambda = OptimizationSettings(OPTS, minPar = [0.01], maxPar = [0.99], initialStepSize = [0.2], objective=:ARL, constraint=:ARL)
        Random.seed!(SD)
        PROC = ProcessSettings(par0 = rand(Beta(10,10), 1), ARL0 = ARL0, delta=[delta], seed=seed)
        settings = Settings(OPTSLambda, PROC)
        filename = savename(settings, "arl-arl.jld2")
        filepath = datadir(simsDir, "EWMA-median", filename)
        opt_arl_arl = spsaCL(settings, runEWMA)
        if isfile(filepath)
            continue
        else
            # Optimize arl-median
            opt_arl_arl = spsaCL(settings, runEWMA)
            saveobj = Dict("delta" => delta, "ARL0" => ARL0,
            "lambda" => opt_arl_arl[:opt][1],
            "type" => "arl-arl"
            )
            safesave(filepath, saveobj)
        end
    end
end
