using DrWatson
@quickactivate "StochApprox"
using Revise
using saControlLimits
import saControlLimits: spsaCL
using StatsBase
using ArgParse
using Random, Distributions
using DataFrames
using JLD2
using TimerOutputs

include(srcdir("settings", "settings.jl"))

DrWatson.default_allowed(c) = (Real, String, Symbol, Vector)
DrWatson.savename(S::Settings, suffix::String) = savename(struct2ntuple(S.PROC), suffix)

argTbl = ArgParseSettings()
@add_arg_table argTbl begin
    "--seed", "-s"
        help = "Simulation seed"
end

parsedArgs = parse_args(ARGS, argTbl)
SD = parse(Int64, parsedArgs["seed"])

simsDir = "gavg"

abstol = 1e-05
NminSPSA = 300

saCL(0.1, (h,m,d) -> runCUSUM(h,m,d, par=1.0), 100, verbose=false)     #? First run for compilation
time()      #? First run for compilation
# ---- CUSUM ---- #
for gavg in [5, 10, 25, 50, 100, 125, 150, 200, 300, 500]
    OPTS = OptimizationSettings(
        hStart = 0.001,
        nAdaptiveGain = 20,
        initialStepSize = [0.2],
        expNumberLossEval = NminSPSA,
        NminSPSA = NminSPSA,
        NfixedSPSA = 100,
        gg = 0.05,
        abstol = abstol,
        gavg = gavg,
        NfixedSA = 100,
        maxiterSA=100,
        maxiterSPSA = 100000,
        minPar = [0.01],
        maxPar = [0.99],

    comparison = nothing)
    println("--- Starting CUSUM ---")
    rng = MersenneTwister(SD)
    deltas = 0.5:0.5:3.0
    ARL0 = 370
    minPar = [0.001]
    maxPar = [4.0]
    for delta in deltas
        seed = abs(rand(rng, Int))
        filename = savename(Dict("gavg" => gavg, "delta" => delta, "ARL0" => ARL0, "SD" => SD), "jld2")
        filepath = datadir(simsDir, "CUSUM", filename)
        if !isfile(filepath)
            #? First run for compilation
            par0 = rand(rng, Uniform(0.0, 3.0), 1)
            optH = saCL(0.1, (h,m,d) -> runCUSUM(h,m,d, par=par0), ARL0, verbose=false)

            OPTS_run = OptimizationSettings(OPTS,
                        hStart = optH[:h],
                        initialStepSize = [0.2],
                        minPar = minPar,
                        maxPar = maxPar,
                        #! Put comparison to compare accuracy
                        comparison = nothing
                        )

            PROC = ProcessSettings(par0 = par0, ARL0 = ARL0, delta=delta, seed=seed)
            settings = Settings(OPTS_run, PROC)

            settings_compile = Settings(settings, OPTIM = OptimizationSettings(OPTS_run, maxiterSPSA=1))
            spsaCL(settings_compile, runCUSUM)

            t_spsa = time()
            opt_spsa = spsaCL(settings, runCUSUM)
            dt_spsa = time() - t_spsa

            saveobj = Dict("delta" => delta,
                            "ARL0" => ARL0,
                            "parSPSA" => opt_spsa[:opt][1], 
                            "nIterICspsa" => opt_spsa[:nIterIC] + optH[:iter],
                            "nIterOCspsa" => opt_spsa[:nIterOC],
                            "time_spsa" => dt_spsa,
                            "gavg" => gavg
                            )
            safesave(filepath, saveobj)
        end
    end
end
