library("pacman")
library("devtools")
library("ungeviz")
p_load("dplyr", "ggplot2", "spc", "kableExtra", "reshape2", "tidyr")
theme_set(theme_bw())
MSE = function(true, est) mean((true - est)^2)
BIAS = function(true, est) mean(est - true)

pd <- position_dodge(0.1) # move them .05 to the left and right


readDF = function(DIR){
    df = read.csv(paste0(DIR, "results.csv"))
    dfOut <- df %>%
        group_by(ARL0, delta) %>%
        summarise(
            blow = quantile(lambda, 0.1, na.rm=TRUE),
            q1 = quantile(lambda, 0.25, na.rm=TRUE),
            q2 = quantile(lambda, 0.5, na.rm=TRUE),
            q3 = quantile(lambda, 0.75, na.rm=TRUE),
            bup = quantile(lambda, 0.9, na.rm=TRUE)
        ) %>%
        arrange(delta)
    dfOut$ARL0 = as.factor(dfOut$ARL0)
    dfOut
}


# # ----------------- EWMA simulations ----------------- #
LOADDIR = "/home/dzago/StochApprox/data/output/sims/"
DIR = paste0(LOADDIR, "EWMA/")
dfEWMA = readDF(DIR)

OC = function(l, delta = 0.0, ARL0 = 100){
    h = xewma.crit(l, ARL0, sided="two")
    xewma.arl(l, h, delta, sided="two")
}

OPT = function(delta, ARL0) optimize(OC, lower=0.0, upper=1.0, delta=delta, ARL0=ARL0)$minimum


TMP = as.numeric(levels(dfEWMA$ARL0))[dfEWMA$ARL0]
delta = seq(min(dfEWMA$delta), max(dfEWMA$delta), length = 100)
ARL0 = c()
for(a in unique(TMP)){
    ARL0 = c(ARL0, rep(a, length(delta)))
}
dfAD = data.frame(cbind("delta" = rep(delta, 4), "ARL0" = ARL0))
dfAD$opt = NA
for(i in 1:NROW(dfAD)){
    dfAD$opt[i] = OPT(dfAD$delta[i], dfAD$ARL0[i])
}
dfAD$ARL0 = as.factor(dfAD$ARL0)

p = ggplot() +
    geom_line(aes(x = delta, y = q2, colour = ARL0), data=dfEWMA) +
    geom_ribbon(aes(x = delta, ymin = blow, ymax = bup, fill = ARL0), data=dfEWMA, alpha = 0.2)+
    geom_line(aes(x = delta, y = opt, colour = ARL0), data=dfAD, linetype = "dashed") +
    xlab("shift") + ylab("lambda") 
ggsave(paste0(DIR, "EWMA-opt.jpg"), p, width=1980, height=1080, units="px", dpi=300)

dfEWMA = read.csv(paste0(DIR, "results.csv"))
ARL0 = sort(unique(dfEWMA$ARL0))
deltas = sort(unique(dfEWMA$delta))
grid = expand.grid(ARL0, deltas)
grid$true = apply(grid, 1, function(row) OPT(row[2], row[1]))


# ----- EWMA MEDIAN ----- # 
DIR = paste0(LOADDIR, "EWMA-median/")
dfEWMA_m = read.csv(paste0(DIR, "results.csv"))
dfE_M <- dfEWMA_m %>%
    group_by(ARL0, delta, type) %>%
    summarise(
        lambda= median(lambda),
    ) %>%
    arrange(delta)
dfE_M = spread(dfE_M, type, lambda)
round(as.data.frame(dfE_M[, c(1, 2, 3, 5, 4, 6)]), digits=4)


# ----- BISECTION CUSUM ----- # 
LOADDIR = "/home/dzago/StochApprox/data/output/bisection/"
for(dir in c("CUSUM/", "CUSUM_est/")){
    DIR = paste0(LOADDIR, dir)
    dfC_B = read.csv(paste0(DIR, "results.csv"))
    dfC_B$parTrue = dfC_B$delta / 2
    ylim_spsa = dfC_B %>% group_by(delta) %>% summarize(q10 = quantile(parSPSA, 0.1), q90 = quantile(parSPSA, 0.9))
    ylim_bi = dfC_B %>% group_by(delta) %>% summarize(q10 = quantile(parBisec, 0.1), q90 = quantile(parSPSA, 0.9))
    ylim_nelder = dfC_B %>% group_by(delta) %>% summarize(q10 = quantile(parNelder, 0.1), q90 = quantile(parNelder, 0.9))
    ylim = c(min(ylim_spsa$q10, ylim_bi$q10, ylim_nelder$q10)-0.05, max(ylim_spsa$q90, ylim_bi$q90, ylim_nelder$q90) + 0.05)

    dfC_BLong_par = melt(dfC_B, measure.vars = c("parSPSA", "parBisec", "parNelder"))

    plt = ggplot(dfC_BLong_par, aes(x = as.factor(delta), y = value, fill = variable)) + 
        geom_boxplot(outlier.shape=NA) + 
        scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead")) + 
        geom_point(aes(y = parTrue, alpha = ""), shape=23, fill="orange", show.legend=FALSE) + 
        scale_alpha_manual("True", values = 1) + 
        coord_cartesian(ylim = ylim) + 
        xlab("delta") + ylab("k")
    ggsave(paste0(DIR, "opt.jpg"), plt, width=1980, height=1080, units="px", dpi=300)

    dfC_BLong = melt(dfC_B, measure.vars = c("time_spsa", "time_bisec", "time_nelder"))

    plt = ggplot(dfC_BLong, aes(x = as.factor(delta), y = value, fill = variable)) + 
        geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("Time (s)") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead")) +
        coord_cartesian(ylim = quantile(dfC_BLong$value, c(0.0, 0.99)))
        # scale_y_continuous(limits = quantile(dfC_BLong$value, c(0.0, 0.9)))
    if(dir == "CUSUM_est/"){
        plt = plt +  scale_y_log10()
    }
    ggsave(paste0(DIR, "time.jpg"), plt, width=1980, height=1080, units="px", dpi=300)
}


# ----- BISECTION EWMA ----- #
DIR = paste0(LOADDIR, "EWMA/")
dfE_B = read.csv(paste0(DIR, "results.csv"))

dfELong_RL = melt(dfE_B, measure.vars = c("RLspsa", "RLbisec", "RLnelder"))
plt = ggplot(dfELong_RL, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("OC ARL") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "runLengths.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


dfELong_time = melt(dfE_B, measure.vars = c("time_spsa", "time_bisec", "time_nelder"))
plt = ggplot(dfELong_time, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("Time (s)") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "time.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


# ----- BISECTION MEWMA2 ----- #
DIR = paste0(LOADDIR, "MEWMA2/")
dfM2_B = read.csv(paste0(DIR, "results.csv"))

dfM2Long_RL = melt(dfM2_B, measure.vars = c("RLspsa", "RLbisec", "RLnelder"))
plt = ggplot(dfM2Long_RL, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("OC ARL") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "runLengths.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


dfM2Long_time = melt(dfM2_B, measure.vars = c("time_spsa", "time_bisec", "time_nelder"))
plt = ggplot(dfM2Long_time, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("Time (s)") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "time.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


# ----- BISECTION MEWMA3 ----- #
DIR = paste0(LOADDIR, "MEWMA3/")
dfM3_B = read.csv(paste0(DIR, "results.csv"))

dfM3Long_RL = melt(dfM3_B, measure.vars = c("RLspsa", "RLbisec", "RLnelder"))
plt = ggplot(dfM3Long_RL, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("OC ARL") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "runLengths.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


dfM3Long_time = melt(dfM3_B, measure.vars = c("time_spsa", "time_bisec", "time_nelder"))
plt = ggplot(dfM3Long_time, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + xlab("delta") + ylab("Time (s)") + scale_fill_discrete("method", labels = c("SPSA", "Grid search", "Nelder-Mead"))
ggsave(paste0(DIR, "time.jpg"), plt, width=1980, height=1080, units="px", dpi=300)

# ----- Plot computational time by dimension ----- #
dfELong_time$dim = 1
dfM2Long_time$dim = 2
dfM3Long_time$dim = 3

toKeep = c("delta", "dim", "variable", "value")
df_time_comparison = rbind(dfELong_time[, toKeep], dfM2Long_time[, toKeep], dfM3Long_time[, toKeep])
df_time_comparison$dim = as.factor(df_time_comparison$dim)

# Create a group-means data set
df <- df_time_comparison %>% 
        group_by(delta, dim, variable) %>% 
        summarise(
          time = mean(value),
          q10 = quantile(value, 0.1),
          q90 = quantile(value, 0.9)
        )

plt = ggplot(df, aes(x = delta, y = time, colour = variable)) + 
    geom_line() + 
    geom_point() + 
    geom_errorbar(aes(ymin=q10, ymax=q90), width=.1) +
    ylab("Time (s)") +
    scale_colour_discrete("method", labels=c("SPSA", "Grid search", "Nelder-Mead"))+
    facet_wrap(~dim)

ggsave(paste0(DIR, "dimension-comparison.jpg"), plt, width=1980, height=1080, units="px", dpi=300)

# ----- Plot run lengths by dimension ----- #
dfELong_RL$dim = 1
dfM2Long_RL$dim = 2
dfM3Long_RL$dim = 3

toKeep = c("delta", "dim", "variable", "value")
df_RL_comparison = rbind(dfELong_RL[, toKeep], dfM2Long_RL[, toKeep], dfM3Long_RL[, toKeep])
df_RL_comparison$dim = as.factor(df_RL_comparison$dim)

plt = ggplot(df_RL_comparison, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + 
    ylab("OC ARL") + xlab("delta") +
    scale_colour_discrete("method", labels=c("SPSA", "Grid search", "Nelder-Mead")) +
    facet_wrap(~dim)

ggsave(paste0(DIR, "runlength-comparison.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


dfE_par <- dfE_B %>%
    group_by(delta) %>%
    summarize(l1spsa = median(par1SPSA), l1bisec = median(par1Bisec), l1nelder = median(par1Nelder), time_spsa = median(time_spsa), time_bisec = median(time_bisec), time_nelder = median(time_nelder),  ARL1spsa = median(RLspsa), ARL1bisec = median(RLbisec), ARL1nelder = median(RLnelder))
dfE_par$l2spsa = NA
dfE_par$l2bisec = NA
dfE_par$l2nelder = NA
dfE_par$l3spsa = NA
dfE_par$l3bisec = NA
dfE_par$l3nelder = NA

dfM2_par <- dfM2_B %>%
    group_by(delta) %>%
    summarize(l1spsa = median(par1SPSA), l1bisec = median(par1Bisec), l1nelder = median(par1Nelder), l2spsa = median(par2SPSA), l2bisec = median(par2Bisec), l2nelder = median(par2Nelder), time_spsa = median(time_spsa), time_bisec = median(time_bisec), time_nelder = median(time_nelder), ARL1spsa = median(RLspsa), ARL1bisec = median(RLbisec), ARL1nelder = median(RLnelder))
dfM2_par$l3spsa = NA
dfM2_par$l3bisec = NA
dfM2_par$l3nelder = NA

dfM3_par <- dfM3_B %>%
    group_by(delta) %>%
    summarize(l1spsa = median(par1SPSA), l1bisec = median(par1Bisec), l1nelder = median(par1Nelder), l2spsa = median(par2SPSA), l2bisec = median(par2Bisec), l2nelder = median(par2Nelder), l3spsa = median(par3SPSA), l3bisec = median(par3Bisec), l3nelder = median(par3Nelder), time_spsa = median(time_spsa), time_bisec = median(time_bisec), time_nelder = median(time_nelder), ARL1spsa = median(RLspsa), ARL1bisec = median(RLbisec), ARL1nelder = median(RLnelder))

dfPar = rbind(dfE_par, dfM2_par, dfM3_par)
dfPar$d = c(rep(1,4), rep(2,4), rep(3,4))
dfPar =  dfPar[, c("d","delta","l1spsa","l2spsa","l3spsa","ARL1spsa","time_spsa","l1bisec","l2bisec","l3bisec","ARL1bisec","time_bisec", "l1nelder","l2nelder","l3nelder","ARL1nelder","time_nelder")]
options(knitr.kable.NA = '-')
tab <- kable(dfPar, format="latex", booktabs=TRUE, digits = 3, row.names=FALSE, escape=FALSE, align='c', linesep = "", col.names = c("$d$","$\\delta$", "$\\lambda_1$", "$\\lambda_2$", "$\\lambda_3$", "$\\text{ARL}_1$", "$t$ (s)",
"$\\lambda_1$", "$\\lambda_2$", "$\\lambda_3$", "$\\text{ARL}_1$",  "$t$ (s)",
"$\\lambda_1$", "$\\lambda_2$", "$\\lambda_3$", "$\\text{ARL}_1$",  "$t$ (s)"),
        caption = "", label="MEWMA-comparison"
) %>%
    kable_styling(latex_options = "HOLD_position") %>%
    pack_rows(index = table(dfPar$d), escape=FALSE, bold=FALSE) %>%
    add_header_above(header = c("", "", "SPSA" = 5, "Grid search" = 5, "Nelder-Mead" = 5))
tab

write(tab, paste0(DIR, "summary.tex"))


# ----- RSADA -----#
LOADDIR = "/home/dzago/StochApprox/data/output/sims/"
DIR = paste0(LOADDIR, "RSADA/")
dfRS = read.csv(paste0(DIR, "results.csv"))


dfRS_Long = melt(dfRS, measure.vars = c("ARL1_opt", "ARL1"))
plt = ggplot(dfRS_Long, aes(x = as.factor(delta), y = value, fill = variable)) + 
    geom_boxplot(outlier.shape=NA) + 
    scale_fill_discrete("method", labels = c("SPSA", "Non-optimized")) + 
    xlab("delta") + ylab("OC ARL") + 
    coord_cartesian(ylim = c(0, 475))
ggsave(paste0(DIR, "opt-comparison.jpg"), plt, width=1980, height=1080, units="px", dpi=300)


pacman::p_load("data.table")
setDT(dfRS)
summ = function(x) c("Mean" = mean(x), "SD" = sd(x), "min" = min(x), "q10" = quantile(x, 0.1), "q90" = quantile(x, 0.9), "max" = max(x))
dfRS_tab = dfRS[, as.list(c(summ(parSPSA1), summ(parSPSA2))), by = delta]


tab <- kable(dfRS_tab, format="latex", booktabs=TRUE, digits = 3, row.names=FALSE, escape=FALSE, align='c', linesep = "", col.names = c("$\\delta$", rep(c("Mean", "SD", "min", "$q_{10}$", "$q_{90}$", "max"), 2)),
        caption = "Optimal parameter values of the R-SADA control chart computed via the proposed SPSA approach. ", label="RSADA-opt"
) %>%
    kable_styling(latex_options = "HOLD_position")
tab

write(tab, paste0(DIR, "summary.tex"))



# ----- gavg study on CUSUM ----- #
LOADDIR = "/home/dzago/StochApprox/data/output/gavg/"
DIR = paste0(LOADDIR, "CUSUM/")
df = read.csv(paste0(DIR, "results.csv"))
dfSum <- df %>% group_by(gavg, delta) %>%
    summarize(
                q10par = quantile(parSPSA[!is.na(parSPSA)], 0.1),
                q90par = quantile(parSPSA[!is.na(parSPSA)], 0.9),
                q50par = median(parSPSA[!is.na(parSPSA)]),
                q10time = quantile(time_spsa, 0.1),
                q90time = quantile(time_spsa, 0.9),
                q50time = median(time_spsa),
                mse = MSE(parSPSA[!is.na(parSPSA)], parTrue[!is.na(parSPSA)]),
                bias = BIAS(parSPSA[!is.na(parSPSA)], parTrue[!is.na(parSPSA)]),
                true = mean(parTrue)
                )

dfSum = dfSum[order(dfSum$delta, dfSum$gavg),]

plt = ggplot(dfSum, aes(x = gavg, y = mse)) + 
        geom_point() +
        geom_line() +
        facet_grid(rows=vars(delta), scales="free_y") + 
        xlab("r") + ylab("Mean Squared Error")
ggsave(paste0(DIR, "mse.jpg"), plt, dpi = 300)

plt = ggplot(dfSum, aes(x = gavg, y = q50time)) + 
        geom_point() +
        geom_line() +
        facet_grid(rows=vars(delta), scales="free_y") +
        xlab("r") + ylab("Time (s)")
ggsave(paste0(DIR, "time.jpg"), plt)