using DrWatson
@quickactivate "StochApprox"
using Revise
using saControlLimits
import saControlLimits: spsaCL
using StatsBase
using ArgParse
using Random, Distributions
using DataFrames
using JLD2
using TimerOutputs

include(srcdir("settings", "settings.jl"))

DrWatson.default_allowed(c) = (Real, String, Symbol, Vector)
DrWatson.savename(S::Settings, suffix::String) = savename(struct2ntuple(S.PROC), suffix)

argTbl = ArgParseSettings()
@add_arg_table argTbl begin
    "--seed", "-s"
        help = "Simulation seed"
end

parsedArgs = parse_args(ARGS, argTbl)
SD = parse(Int64, parsedArgs["seed"])

simsDir = "sims"

abstol = 1e-05
NminSPSA = 300
OPTS = OptimizationSettings(
    hStart = 0.001,
    nAdaptiveGain = 20,
    initialStepSize = [0.2],
    expNumberLossEval = NminSPSA,
    NminSPSA = NminSPSA,
    NfixedSPSA = 100,
    gg = 0.05,
    abstol = abstol,
    gavg = 100,
    NfixedSA = 100,
    maxiterSA=100,
    maxiterSPSA = 100000,
    minPar = [0.01],
    maxPar = [0.99],
    comparison = nothing)


# ---- RSADA optimization ---- #
rng = MersenneTwister(SD)
deltas = [0.25, 0.5, 1.0, 2.0]
ARL0 = 370
minPar = [0.01, 0.1]
maxPar = [Inf, 5.0]
for delta in deltas
    seed = abs(rand(rng, Int))
    filename = savename(Dict("delta" => delta, "ARL0" => ARL0, "SD" => SD), "jld2")
    filepath = datadir(simsDir, "RSADA", filename)
    if isfile(filepath)
        continue
    else
        par0 = [0.3, 1.5]
        optH = saCL(10.0, (h,m,d) -> runRSADA(h,m,d, par=par0), ARL0, verbose=true, maxiter = 50000)
        OPTS_run = OptimizationSettings(OPTS,
                    hStart = optH[:h],
                    initialStepSize = [0.1, 0.1],
                    type = [:Numeric, :Numeric],
                    minPar = minPar,
                    maxPar = maxPar,
                    #! Put comparison to compare accuracy
                    comparison = nothing
                    )

        PROC = ProcessSettings(par0 = par0, ARL0 = ARL0, delta=delta, seed=seed)
        settings = Settings(OPTS_run, PROC)

        opt_spsa = spsaCL(settings, runRSADA)

        ARL_unoptimized = mean(runRSADA(optH[:h], par=par0, shift=delta)[:rl] for _ in 1:10^4)
        optH_spsa = saCL(optH[:h], (h,m,d) -> runRSADA(h,m,d, par=opt_spsa[:opt]), ARL0, verbose=true, maxiter=50000)
        ARL_optimized = mean(runRSADA(optH_spsa[:h], par=opt_spsa[:opt], shift=delta)[:rl] for _ in 1:10^4)
        saveobj = Dict("delta" => delta,
                        "ARL0" => ARL0,
                        "parSPSA1" => opt_spsa[:opt][1], 
                        "parSPSA2" => opt_spsa[:opt][2], 
                        "ARL1" => ARL_unoptimized,
                        "ARL1_opt" => ARL_optimized,
                        "nIterICspsa" => opt_spsa[:nIterIC] + optH[:iter],
                        "nIterOCspsa" => opt_spsa[:nIterOC]
                        )
        safesave(filepath, saveobj)
    end
end
