function bisectionCL(hMin, hMax, rlsim, ARL0; maxiter=50, nsims = 10000, abstol = 1e-06, RLtol = 3.0, verbose = false, trunc=50*ARL0)
    RLs = zeros(nsims)
    E_RL = h = 0.0
    hOld = hMax + 1
    i = 0
    if verbose
        println("Starting optimization with endpoints [$(hMin), $(hMax)]")
    end
    while i < maxiter
        i = i + 1       
        h = (hMin + hMax) / 2
        if verbose
            print("i: $(i)/$(maxiter),\th: $(h)")
        end
        for j in 1:nsims
            RLs[j] = rlsim(h, trunc, 0.0)[:rl]
            # while RLs[j] == trunc
            #     RLs[j] = rlsim(h, trunc, 0.0)[:rl]
            # end
        end
        E_RL = mean(RLs)
        if verbose
            println("\tE[RL] = $(E_RL)")
        end
        if E_RL > ARL0
            hMax = h
        else
            hMin = h
        end
        if abs(E_RL - ARL0) < RLtol
            break
        end
        if abs(hOld - h) < abstol
            break
        end
        hOld = h
    end
    @show h
    return (h = h, iter = i)
end


function bisectionOpt(minpar, maxpar, rlsim, ARL0; shift, x::Union{Nothing, Vector{Float64}} = nothing, m = 10, maxiter=1000, nsim = 10000, eps3 = 1e-03, verbose = false, seed = 1234)
    Random.seed!(seed)
    maxPar = deepcopy(maxpar)
    minPar = deepcopy(minpar)
    p = length(minPar)
    gridVec = Vector{Vector{Float64}}(undef, p)
    step = Vector{Float64}(undef, p)
    BIGINT = 10^5
    RLs = zeros(nsim)
    RL = 0.0
    i = 0
    nIterIC = 0
    nIterOC = 0
    bestRL = 10000
    parCurrent = zeros(p)
    parOld = zeros(p)
    while i < maxiter
        i = i + 1       
        parOld = parCurrent
        step = [maxPar[w] - minPar[w] for w in 1:p]
        # @show step./m, gridVec, minPar
        for w in 1:p
            gridVec[w] = [minPar[w] + step[w] * j/m for j in 0:m]
        end

        grid = Base.product(gridVec...)
        cnt = 0
        seedRL = abs(rand(Int))
        for parTuple in grid
            cnt += 1
            par = collect(parTuple)
            if isnothing(x)
                opt = saCL(0.1, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=par), ARL0, verbose = false)
            else
                opt = saCL(0.1, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=par, x=x), ARL0, verbose = false)
            end
            hOpt = opt[:h]
            nIterIC += opt[:iter] 
            for j in 1:nsim
                if isnothing(x)
                    RLs[j] = rlsim(hOpt, BIGINT, 0.0; par=par, shift=shift, seed=seedRL+j)[:rl]
                else
                    RLs[j] = rlsim(hOpt, BIGINT, 0.0; par=par, shift=shift, seed=seedRL+j, x=x)[:rl]
                end
                nIterOC += 1
            end
            RL = mean(RLs)
            # @show par, RL
            if cnt == 1
                bestRL = RL       
                parCurrent = par
            else
                if RL < bestRL
                    bestRL = RL
                    parCurrent = par
                end
            end
        end

        for w in 1:p
            if parCurrent[w] == maxPar[w]
                parCurrent[w] -= step[w]/m
            elseif parCurrent[w] == minPar[w]
                parCurrent[w] += step[w]/m
            end
            maxPar[w] = parCurrent[w] + step[w]/m
            minPar[w] = parCurrent[w] - step[w]/m
        end

        if verbose
            println("i: $(i)/$(maxiter),\tparCurrent: $(parCurrent)")
        end

        if norm(parCurrent - parOld) < eps3
            break
        end
    end
    return (opt = parCurrent, iter = i, nIterIC = nIterIC, nIterOC = nIterOC)
end
export bisectionOpt