using Random
using Optim
using NLopt


function NelderMeadOpt(par0, minpar, maxpar, rlsim, ARL0; maxiter = 1000, shift, ncond = 10000, eps = 1e-05, seed = abs(rand(Int)))
    Random.seed!(seed)
    function optimRL(par, grad::Vector)
        # @show par
        opt = saCL(0.1, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=par), ARL0, verbose = false)
        return mean(rlsim(opt[:h], 10^5, 0.0; par=par, shift=shift, seed=abs(rand(Int)))[:rl] for _ in 1:ncond)
    end
    
    # inner_optimizer = NelderMead()
    # opt = Optim.optimize(optimRL, minpar, maxpar, par0, Fminbox(inner_optimizer), Optim.Options(x_tol = eps, iterations = 40, outer_iterations = 5, show_trace=true))
    # return opt

    opt = Opt(:LN_NELDERMEAD, length(par0))
    opt.lower_bounds = minpar
    opt.upper_bounds = maxpar
    opt.xtol_rel = eps
    opt.maxeval = maxiter
    opt.min_objective = optimRL
    (minf, minx, ret) = NLopt.optimize(opt, par0)
    return (minf, minx, ret)
end
export NelderMeadOpt


function EvoOpt(par0, minpar, maxpar, rlsim, ARL0; maxiter = 200, shift, ncond = 10000, eps = 1e-05)
    function optimRL(par, grad::Vector)
        opt = saCL(0.1, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=par), ARL0, verbose = false)
        return mean(rlsim(opt[:h], 10^5, 0.0; par=par, shift=shift, seed=abs(rand(Int)))[:rl] for _ in 1:ncond)
    end
    
    opt = Opt(:LN_ESCH, length(par0))
    opt.lower_bounds = minpar
    opt.upper_bounds = maxpar
    opt.xtol_rel = eps
    opt.maxeval = maxiter
    opt.min_objective = optimRL
    (minf, minx, ret) = NLopt.optimize(opt, par0)
    return (minf, minx, ret)
end
export EvoOpt
