# similar(x::Float64)
function KW_singleiteration(n, eps, rlsim, ARL0, parOpt, hOptPlus, hOptMinus, ngrad, delta)
    a_n = 1.0 / n
    c_n = n^(-1/3)

    parPlus = max(eps, parOpt + c_n)
    hOptPlus = saCL(hOptPlus, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parPlus, shift = 0.0), ARL0, maxiter = 1e05, verbose = false)[:h]

    parMinus = max(eps, parOpt - c_n)
    hOptMinus = saCL(hOptMinus, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parMinus, shift = 0.0), ARL0, maxiter = 1e05, verbose = false)[:h]
    score = 0.0
    for g in 1:ngrad
        rlPlus  = rlsim(hOptPlus, 10*ARL0, 0.0, par = parPlus, shift=delta)[:rl]
        rlMinus = rlsim(hOptMinus, 10*ARL0, 0.0, par = parMinus, shift=delta)[:rl]
        score = score + ((rlPlus - rlMinus)/(2.0*c_n) - score)/g
    end
    @show parPlus, parMinus, score
    (score = score, a_n = a_n)
end



function saCLnested(par0, h0, rlsim, ARL0, delta; maxiter = 10000, z = 3.0, gamma = 0.1, Nmin = 100, Nfixed = 100, ngrad = 100)
    hOptPlus = deepcopy(h0)
    hOptMinus = deepcopy(h0)
    parOpt = par0
    parPR = par0
    eps = 0.001
    pivot = (par0 + eps)/2.0
    s2 = 0.0
    v = (z/gamma)^2
    n = 0
    while n < maxiter
        n = n+1
        @show n, parOpt, parPR
        # @show a_n, c_n, score 
        iter = KW_singleiteration(n, eps, rlsim, ARL0, parOpt, hOptPlus, hOptMinus, ngrad, delta)
        score = iter[:score]
        a_n = iter[:a_n]
        
        newPar = parOpt - a_n * score
        parOpt = max(eps, newPar)
        
        if parOpt == eps
            parOpt = pivot
            pivot = par0 / log(exp(1)+n)
        end

        if n > Nfixed
            parPR = parPR + (parOpt - parPR) / (n - Nfixed)
            s2 = s2 + ((score * score)/ngrad - s2) / (n - Nfixed)
        end

        @show v * maximum(s2)
        if (n > Nfixed + Nmin) && (n > v * maximum(s2))
            break
        end
    end

    hOpt = saCL((hOptPlus + hOptMinus)/2, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parPR, shift = 0.0), ARL0, maxiter = 1e05, verbose = false)[:h]
    return (h = hOpt, par = parPR, iter = n)
end


function saCLnestedBisection(par0, h0, rlsim, ARL0, delta; maxiter=10000, z=3.0, gamma=0.1, abstol=0.001, minpar=0.01, maxpar=6, nsims=10000)
    # hOptMax = deepcopy(h0)
    # hOptMax = saCL(hOptMax, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=maxpar, shift=0.0), ARL0, maxiter=1e05, verbose=false)[:h]

    # rlMax = sum([rlsim(hOptMax, 10 * ARL0, 0.0, par=maxpar, shift=delta)[:rl] for _ in 1:nsims])/nsims

    # hOptMin = deepcopy(h0)
    # hOptMin = saCL(hOptMin, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=minpar, shift=0.0), ARL0, maxiter=1e05, verbose=false)[:h]

    # rlMin = sum([rlsim(hOptMin, 10 * ARL0, 0.0, par=minpar, shift=delta)[:rl] for _ in 1:nsims])/nsims

    parNew = (minpar + maxpar) / 2
    hOptNew = saCL(deepcopy(h0), (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parNew, shift=0.0), ARL0, maxiter=1e05, verbose=false)[:h]

    rlNew = sum([rlsim(hOptNew, 10 * ARL0, 0.0, par=parNew, shift=delta)[:rl] for _ in 1:nsims])/nsims

    parPlus = parMinus = 0.0
    rlPlus = rlMinus = 0
    hOptPlus = hOptMinus = 0.0
    while true
        parPlus = (parNew + maxpar)/2
        parMinus = (parNew + minpar)/2

        hOptPlus = saCL(hOptPlus, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parPlus, shift=0.0), ARL0, maxiter=1e05, verbose=false)[:h]

        rlPlus = sum([rlsim(hOptPlus, 10 * ARL0, 0.0, par=parPlus, shift=delta)[:rl] for _ in 1:nsims])/nsims

        hOptMinus = saCL(hOptMinus, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parMinus, shift=0.0), ARL0, maxiter=1e05, verbose=false)[:h]

        rlMinus = sum([rlsim(hOptMinus, 10 * ARL0, 0.0, par=parMinus, shift=delta)[:rl] for _ in 1:nsims])/nsims

        if rlPlus < rlMinus
            minpar = parNew
        else
            maxpar = parNew
        end
        parNew = (minpar + maxpar) / 2

        @show minpar, maxpar
        absErr = abs(minpar - maxpar)
        # @show absErr, abstol
        cond = absErr < abstol
        cond && break
    end

    println("Convergence.")
    parOpt = (minpar + maxpar)/2
    hOpt = saCL(hOptPlus, (h, maxrl, deltaSA) -> rlsim(h, maxrl, deltaSA; par=parOpt, shift = 0.0), ARL0, maxiter = 1e05, verbose = false)[:h]

    return (h = hOpt, par = parOpt)
end
