using saControlLimits
using Random

# Function to simulate a chart run length
function runCusum(h, maxrl = 100000, deltaSA=0.0; par = 2.0, shift = 0.0)
    i = 0
    R = 0.0
    p = length(h)

    rl = rlPlus = rlMinus = 0
    notDoneRl = true
    notDoneP = notDoneM = (deltaSA > 0.0)
    notDone = notDoneRl + notDoneM + notDoneP
    # @show par, shift
    while i < maxrl && (notDone > 0)
        i = i+1
        y = randn() + shift
        R = max(0.0, R + y - par)
        if notDoneRl && R > h
            notDoneRl = false
            rl = i
        end
        if notDoneP && R > h + deltaSA
            notDoneP = false
            rlPlus = i
        end
        if notDoneM && R > h - deltaSA
            notDoneM = false
            rlMinus = i
        end
        notDone = notDoneRl + notDoneM + notDoneP
    end

    if deltaSA == 0.0
        rlPlus = rl
        rlMinus = rl
    end
    return (rl = rl, rlPlus = rlPlus, rlMinus = rlMinus)
end



@testset "Single chart" begin
    Random.seed!(123)
    ARL0 = 200
    opt = saCL(1.0, runSim, ARL0, maxiter = 1e05, verbose = true, adaptive=true)

    h = opt[:h]
    ncond = 10000
    rls = [runSim(h, 1e04, 0.0)[:rl] for _ in 1:ncond]
    sum(rls)/ncond
    @test isa(opt, NamedTuple)
    @test length(opt) == 2
    @test opt[:iter] < 4e05
    htrue = 0.5823812   # R> xcusum.crit(2.0, 200)
    @test (0.98)*htrue < opt[:h] < 1.02*htrue

    par0 = 2.0
    h0 = 1.0
    delta = 1.0
    optJoint = saCLnested(par0, h0, runSim, ARL0, delta)

    optJoint2 = saCLnestedBisection(par0, h0, runSim, ARL0, delta, minpar=0.001, maxpar=2.0)
end
