using Parameters

abstract type AbstractSettings end

@with_kw struct OptimizationSettings <: AbstractSettings
    hStart::Float64 = 1e-05
    minPar::Vector{Float64} = [0.001]
    maxPar::Vector{Float64} = [Inf]
    initialStepSize::Vector{Float64} = [0.05]
    expNumberLossEval::Int = 100
    nAdaptiveGain::Int = 50
    NfixedSPSA::Int = 200
    NminSPSA::Int = 200
    gavg::Int = 100
    NfixedSA::Int = 50
    NminSA::Int = 100
    maxiterSA::Int = 100
    maxiterSPSA::Int = 10000
    comparison::Union{Nothing, Int} = nothing
    gg::Float64 = 0.2
    abstol::Float32 = 1e-03
    type::Vector{Symbol} = [:Numeric for _ in 1:length(minPar)]
    objective::Symbol = :ARL
    constraint::Symbol = :ARL

    @assert length(minPar) == length(maxPar)
    @assert length(minPar) == length(initialStepSize)
    @assert length(minPar) == length(type)
end

@with_kw struct ProcessSettings{A,D} <: AbstractSettings
    par0::Vector{Float64}
    ARL0::A
    delta::D
    seed::Int = 12345           # Seed for replicability
end


@with_kw struct Settings <: AbstractSettings
    OPTIM::OptimizationSettings
    PROC::ProcessSettings
end


function spsaCL(settings::Settings, runSim::Function)
    OPTS = settings.OPTIM
    PROC = settings.PROC
    spsaCL(PROC.par0,
            runSim,
            PROC.ARL0,
            PROC.delta,
            hStart = OPTS.hStart,
            nAdaptiveGain = OPTS.nAdaptiveGain,
            NfixedSPSA = OPTS.NfixedSPSA,
            NminSPSA = OPTS.NminSPSA,
            initialStepSize=OPTS.initialStepSize,
            expNumberLossEval = OPTS.expNumberLossEval,
            maxPar = OPTS.maxPar,
            minPar = OPTS.minPar,
            gavg = OPTS.gavg,
            maxiterSA = OPTS.maxiterSA,
            maxiterSPSA = OPTS.maxiterSPSA,
            seed = PROC.seed,
            comparison = OPTS.comparison,
            gg = OPTS.gg,
            abstol = OPTS.abstol,
            type = OPTS.type,
            verbose = false,
            objective = OPTS.objective,
            constraint = OPTS.constraint
            )
end
